
/*
 * MGL graphics library
 * Copyright (C) 1999-2000, ing. Marek Gayer

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.

 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <stdio.h>
//#include <malloc.h>
// free bsd and probably others lacks malloc.h

#include <mgl.h>

int _mallocated;
int _mtotal;

int _allocCount;
int _freeCount;

//#define _mfree ((_mtotal<_mallocated) ? 0 : (_mtotal - _mallocated))

#ifdef _WIN32
#include <malloc.h>
#else
int _msize (void *size)
{
  return 0;
}
#endif

void _mexit (int requestedMemorySize)
{
//  unsigned long mtotal=_mtotal;
  unsigned long mallocated=_mallocated;
//  unsigned long mfree=_mfree;
  char exitStr [200];

  if (exitStr)
  {
    puts (exitStr);

    sprintf (exitStr,
    "Not enough memory !"
    "Requested memory block size : %8i bytes\n"
    "Memory allocated            : %8lu bytes\n"
    "\n", requestedMemorySize, mallocated);
//  mfree, mtotal);
//  "Memory free                 : %8lu bytes\n"
//  "Memory available total      : %8lu bytes\n";
    mglError (MGL_ERROR_NOT_ENOUGH_MEMORY, exitStr);
  }
}

/*
void _mdone (void)
{
  _minit ();
}

void _minit (void)
{
  if (_mtotal == 0)
  {
    _mtotal = DPMI_maxAvail;
  }
}
*/

/*
void *_alloc (int size)
{
  void *mem;

  mem = malloc (size);
  if (mem)
  {
    _allocCount++;
    size = _msize (mem);
    _mallocated += size;
  }
  return mem;
}
*/


void *_malloc (int size)
{
  void *mem;

  mem = malloc (size);
  if (mem)
  {
    _allocCount++;
    size = _msize (mem);
    _mallocated += size;
  }
  else
  {
    _mexit (size);
  }
  return mem;
}

void *_calloc (int n, int size)
{
  void *mem;

  mem = calloc (n, size);
  if (mem)
  {
    _allocCount++;
    size = _msize (mem);
    _mallocated += size;
  }
  else
  {
    _mexit (n*size);
  }
  return mem;
}

void *_realloc (void *mem, int newsize)
{
  int size;

  if (mem)
  {
    size = _msize (mem);
    mem = realloc (mem, newsize);
    if (mem == NULL)
    {
      _mexit (newsize - size);
      return NULL;
    }
    size -= _msize (mem);
    _mallocated -= size;
    return mem;
  }
  else
  {
    _mexit (0);
    return NULL;
  }
}

void _free (void *mem)
{
  int size;

  if (mem)
  {
    _freeCount++;

    size = _msize (mem);
    if (size != -1)
    {
      free (mem);
      _mallocated -= size;
    }
  }
}

